package at.jku.students.redblacktree;

import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;

import at.jku.ssw.Iterator;

public class RedBlackTreeMain {
	private static final Path DUMP_DIR = Paths.get(".");

	public static void main(String[] args) throws IOException {
		RedBlackTree t = new RedBlackTree();
		animateFromString(t, "OSTEREI");
		System.out.println(t.toString());
		printInOrder(t);
		printPreOrder(t);
		printPostOrder(t);
	}

	private static void animateFromString(RedBlackTree t, String s) throws IOException {
		for (int i = 0; i < s.length(); i++) {
			t.insert(s.charAt(i));
			final Path dstFile = DUMP_DIR.resolve(String.format("step%d.dot", i));
			try (FileWriter fw = new FileWriter(dstFile.toFile(), false)) {
				fw.write(t.makeDot());
			}
		}
	}

	private static void printInOrder(RedBlackTree tree) {
		System.out.print("  inorder: ");
		printIterator(new InOrderTreeIterator(tree));
	}

	private static void printPreOrder(RedBlackTree tree) {
		System.out.print(" preorder: ");
		printIterator(new PreOrderTreeIterator(tree));
	}

	private static void printPostOrder(RedBlackTree tree) {
		System.out.print("postorder: ");
		printIterator(new PostOrderTreeIterator(tree));
	}

	private static void printIterator(Iterator it) {
		System.out.print('[');
		if (it.hasNext()) {
			System.out.print(it.next());
			while (it.hasNext()) {
				System.out.print(", ");
				System.out.print(it.next());
			}
		}
		System.out.println(']');
	}

}
